@@ -67,13 +67,13 @@
 #'   However, sampling speed is currently not improved or even
 #'   slightly decreased.
 #' @param cov_ranef A list of matrices that are proportional to the 
-#'   (within) covariance structure of the random effects. 
+#'   (within) covariance structure of the group-level effects. 
 #'   The names of the matrices should correspond to columns 
 #'   in \code{data} that are used as grouping factors. 
 #'   All levels of the grouping factor should appear as rownames 
 #'   of the corresponding matrix. This argument can be used,
 #'   among others, to model pedigrees and phylogenetic effects.
-#' @param ranef A flag to indicate if random effects 
+#' @param ranef A flag to indicate if group-level effects 
 #'   for each level of the grouping factor(s) 
 #'   should be saved (default is \code{TRUE}). 
 #'   Set to \code{FALSE} to save memory. 
@@ -181,7 +181,14 @@
 #'   Its documentation contains detailed information 
 #'   on how to correctly specify priors. To find out on 
 #'   which parameters or parameter classes priors can be defined, 
-#'   use \code{\link[brms:get_prior]{get_prior}}. \cr
+#'   use \code{\link[brms:get_prior]{get_prior}}.
+#'   Default priors are chosen to be non or very weakly informative 
+#'   so that their influence on the results will be negligable and
+#'   you don't have to worry about them.
+#'   However, after getting more familiar with Bayesian statistics, 
+#'   I recommend you to start thinking about reasonable informative
+#'   priors for your model parameters: Nearly always, there is at least some
+#'   prior information available that can be used to improve your inference.
 #'   
 #'   \bold{Adjusting the sampling behavior of \pkg{Stan}}
 #'   
@@ -225,8 +232,8 @@
 #' fit1 <- brm(count ~ log_Age_c + log_Base4_c * Trt_c  
 #'               + (1|patient) + (1|obs), 
 #'             data = epilepsy, family = poisson(), 
-#'             prior = c(set_prior("student_t(5,0,10)", class = "b"),
-#'                       set_prior("cauchy(0,2)", class = "sd")))
+#'             prior = c(prior(student_t(5,0,10), class = b),
+#'                       prior(cauchy(0,2), class = sd)))
 #' ## generate a summary of the results
 #' summary(fit1)
 #' ## plot the MCMC chains as well as the posterior distributions
@@ -268,8 +275,8 @@
 #' x <- rnorm(100)
 #' y <- rnorm(100, mean = 2 - 1.5^x, sd = 1)
 #' fit5 <- brm(y ~ a1 - a2^x, nonlinear = a1 + a2 ~ 1,
-#'             prior = c(set_prior("normal(0, 2)", nlpar = "a1"),
-#'                       set_prior("normal(0, 2)", nlpar = "a2")))
+#'             prior = c(prior(normal(0, 2), nlpar = a1),
+#'                       prior(normal(0, 2), nlpar = a2)))
 #' summary(fit5)
 #' plot(marginal_effects(fit5), ask = FALSE)
 #' 
@@ -338,8 +345,7 @@ brm <- function(formula, data = NULL, family = gaussian(),
     family <- check_family(family)
     formula <- update_formula(formula, data = data, family = family, 
                               nonlinear = nonlinear, partial = partial)
-    et <- extract_time(autocor$formula)  
-    ee <- extract_effects(formula, family = family, et$all)
+    ee <- extract_effects(formula, family = family, autocor = autocor)
     if (is.null(dots$data.name)) {
       data.name <- substr(Reduce(paste, deparse(substitute(data))), 1, 50)
     } else {
@@ -347,7 +353,7 @@ brm <- function(formula, data = NULL, family = gaussian(),
       dots$data.name <- NULL
     }
     # see data-helpers.R
-    data <- update_data(data, family = family, effects = ee, et$group) 
+    data <- update_data(data, family = family, effects = ee)
     # see priors.R
     prior <- check_prior(prior, formula = formula, data = data, 
                          family = family, sample_prior = sample_prior, 
