@@ -16,8 +16,9 @@
 #'   \code{exponential}, \code{weibull}, \code{Beta}, \code{von_mises},
 #'   \code{categorical}, \code{cumulative}, \code{cratio}, \code{sratio}, 
 #'   \code{acat}, \code{hurdle_poisson}, \code{hurdle_negbinomial}, 
-#'   \code{hurdle_gamma}, \code{zero_inflated_binomial},
-#'   \code{zero_inflated_beta}, \code{zero_inflated_negbinomial}, 
+#'   \code{hurdle_gamma}, \code{hurdle_lognormal}, 
+#'   \code{zero_inflated_binomial}, \cr \code{zero_inflated_beta},
+#'   \code{zero_inflated_negbinomial}, 
 #'   and \code{zero_inflated_poisson}.
 #' @param link A specification for the model link function. 
 #'   This can be a name/expression or character string. 
@@ -39,14 +40,14 @@
 #'   \code{exponential}, \code{lognormal}, and \code{inverse.gaussian} can be used 
 #'   (among others) for survival regression.
 #'   Families \code{hurdle_poisson}, \code{hurdle_negbinomial}, \code{hurdle_gamma}, 
-#'   \code{zero_inflated_poisson}, and \cr
-#'   \code{zero_inflated_negbinomial} combined with the 
-#'   \code{log} link, and  \code{zero_inflated_binomial} with the \code{logit} link, 
-#'   allow to estimate zero-inflated and hurdle models. These models 
-#'   can be very helpful when there are many zeros in the data that cannot be explained 
-#'   by the primary distribution of the response. Family \code{hurdle_gamma} is 
-#'   especially useful, as a traditional \code{Gamma} model cannot be reasonably 
-#'   fitted for data containing zeros in the response. 
+#'   \code{hurdle_lognormal}, \code{zero_inflated_poisson},
+#'   \code{zero_inflated_negbinomial}, \code{zero_inflated_binomial}, and
+#'   \code{zero_inflated_beta} allow to estimate zero-inflated and hurdle models. 
+#'   These models can be very helpful when there are many zeros in the data 
+#'   that cannot be explained by the primary distribution of the response. 
+#'   Families \code{hurdle_lognormal} and \code{hurdle_gamma} are 
+#'   especially useful, as traditional \code{lognormal} or \code{Gamma}
+#'   models cannot be reasonably fitted for data containing zeros in the response.
 #'   
 #'   In the following, we list all possible links for each family.
 #'   The families \code{gaussian}, and \code{student},
@@ -65,7 +66,8 @@
 #'   \code{inverse}, \code{identity} and \code{log}; 
 #'   families \code{hurdle_poisson}, \code{hurdle_negbinomial},
 #'   \code{hurdle_gamma}, \code{zero_inflated_poisson}, and
-#'   \code{zero_inflated_negbinomial} the link \code{log}. 
+#'   \code{zero_inflated_negbinomial} the link \code{log};
+#'   family \code{hurdle_lognormal} the link \code{identity}.
 #'   The first link mentioned for each family is the default.     
 #'   
 #'   Please note that when calling the \code{\link[stats:family]{Gamma}} 
@@ -122,8 +124,9 @@ brmsfamily <- function(family, link = NULL) {
     "inverse.gaussian", "beta", "von_mises",
     "cumulative", "cratio", "sratio", "acat",
     "hurdle_poisson", "hurdle_negbinomial", "hurdle_gamma",
-    "zero_inflated_poisson", "zero_inflated_negbinomial",
-    "zero_inflated_binomial", "zero_inflated_beta")
+    "hurdle_lognormal", "zero_inflated_poisson", 
+    "zero_inflated_negbinomial", "zero_inflated_binomial", 
+    "zero_inflated_beta")
   if (!family %in% ok_families) {
     stop(family, " is not a supported family. Supported families are: \n",
          paste(ok_families, collapse = ", "), call. = FALSE)
@@ -145,12 +148,14 @@ brmsfamily <- function(family, link = NULL) {
     ok_links <- c("logit")
   } else if (is.skewed(family)) {
     ok_links <- c("log", "identity", "inverse")
-  } else if (family %in% "lognormal") {
+  } else if (is.lognormal(family)) {
     ok_links <- c("identity", "inverse")
+  } else if (family %in% "hurdle_lognormal") {
+    ok_links <- c("identity")
   } else if (family %in% "von_mises") {
     ok_links <- c("tan_half")
   } else if (is.hurdle(family) || is.zero_inflated(family)) {
-    # does not include zi_binomial or zi_beta
+    # does not include zi_binomial, zi_beta, or hu_lognormal
     ok_links <- c("log")
   }
   # non-standard evaluation of link
@@ -263,6 +268,13 @@ hurdle_gamma <- function(link = "log") {
   .brmsfamily("hurdle_gamma", link = link, slink = slink)
 }
 
+#' @rdname brmsfamily
+#' @export
+hurdle_lognormal <- function(link = "identity") {
+  slink <- substitute(link)
+  .brmsfamily("hurdle_lognormal", link = link, slink = slink)
+}
+
 #' @rdname brmsfamily
 #' @export
 zero_inflated_beta <- function(link = "logit") {
@@ -408,7 +420,7 @@ is.lognormal <- function(family) {
   if (is(family, "family")) {
     family <- family$family
   }
-  family %in% "lognormal"
+  family %in% c("lognormal")
 }
 
 is.count <- function(family) {
@@ -426,7 +438,7 @@ is.hurdle <- function(family, zi_beta = TRUE) {
   }
   # zi_beta is technically a hurdle model
   family %in% c("hurdle_poisson", "hurdle_negbinomial", "hurdle_gamma",
-                if (zi_beta) "zero_inflated_beta")
+                "hurdle_lognormal", if (zi_beta) "zero_inflated_beta")
 }
 
 is.zero_inflated <- function(family, zi_beta = FALSE) {
@@ -478,7 +490,7 @@ use_real <- function(family) {
   }
   is.linear(family) || is.skewed(family) || 
     family %in% c("lognormal", "inverse.gaussian", "beta", "von_mises",
-                  "zero_inflated_beta", "hurdle_gamma")
+                  "zero_inflated_beta", "hurdle_gamma", "hurdle_lognormal")
 }
 
 use_int <- function(family) {
@@ -549,8 +561,12 @@ has_sigma <- function(family, effects = NULL, autocor = cor_arma(),
   #  effects: list returned by extract_effects
   #  autocor: object of class cor_arma
   #  incmv: should MV (linear) models be treated as having sigma? 
+  if (is(family, "family")) {
+    family <- family$family
+  }
+  is_lognormal <- family %in% c("lognormal", "hurdle_lognormal")
   has_se <- !is.null(effects$se)
-  out <- (is.linear(family) || is.lognormal(family)) && 
+  out <- (is.linear(family) || is_lognormal) && 
          (!has_se || use_cov(autocor)) && !is(autocor, "cov_fixed")
   if (!incmv) {
     is_multi <- is.linear(family) && length(effects$response) > 1L
