@@ -36,14 +36,13 @@ make_stancode <- function(formula, data = NULL, family = gaussian(),
                             partial = partial, nonlinear = nonlinear)
   autocor <- check_autocor(autocor)
   threshold <- match.arg(threshold)
-  et <- extract_time(autocor$formula)  
-  ee <- extract_effects(formula, family = family, et$all)
+  ee <- extract_effects(formula, family = family, autocor = autocor)
   prior <- check_prior(prior, formula = formula, data = data,
                        family = family, autocor = autocor,
                        threshold = threshold)
   prior_only <- identical(sample_prior, "only")
   sample_prior <- if (prior_only) FALSE else sample_prior
-  data <- update_data(data, family = family, effects = ee, et$group)
+  data <- update_data(data, family = family, effects = ee)
   
   # flags to indicate the family type
   is_categorical <- is.categorical(family)
