@@ -246,11 +246,11 @@ test_that("make_standata handles covariance matrices correctly", {
   B <- diag(1, 4)
   expect_error(make_standata(count ~ Trt_c + (1|visit), data = epilepsy,
                              cov_ranef = list(visit = B)),
-               "rownames are required")
+               "Row names are required")
   B <- structure(diag(1, 4), dimnames = list(2:5, NULL))
   expect_error(make_standata(count ~ Trt_c + (1|visit), data = epilepsy,
                              cov_ranef = list(visit = B)),
-               "rownames .* do not match")
+               "Row names .* do not match")
   B <- structure(diag(1:5), dimnames = list(c(1,5,2,4,3), NULL))
   expect_equivalent(make_standata(count ~ Trt_c + (1|visit), data = epilepsy,
                              cov_ranef = list(visit = B))$Lcov_1,
@@ -311,7 +311,7 @@ test_that("make_standata correctly prepares data for monotonic effects", {
   
   prior <- c(set_prior("dirichlet(c(1,0.5,2))", class = "simplex", coef = "x2"))
   expect_error(make_standata(y ~ monotonic(x1 + x2), data = data, prior = prior),
-               "Invalid dirichlet prior for the simplex of x2", fixed = TRUE)
+               "Invalid dirichlet prior for the simplex of 'x2'", fixed = TRUE)
 })
 
 test_that("make_standata returns fixed residual covariance matrices", {
@@ -333,15 +333,15 @@ test_that("make_standata returns data for bsts models", {
 
 test_that("make_standata returns data for GAMMs", {
   dat <- data.frame(y = rnorm(10), x1 = rnorm(10), x2 = rnorm(10),
-                    z = rnorm(10), g = rep(1:2, 5))
-  standata <- make_standata(y ~ s(x1) + z + s(x2), data = dat)
+                    x3 = rnorm(10), z = rnorm(10), g = rep(1:2, 5))
+  standata <- make_standata(y ~ s(x1) + z + s(x2, by = x3), data = dat)
   expect_equal(standata$nb_1, 1)
   expect_equal(as.vector(standata$knots_2), 8)
   expect_equal(dim(standata$Zs_1_1), c(10, 8))
   expect_equal(dim(standata$Zs_2_1), c(10, 8))
   
   standata <- make_standata(y ~ lp, data = dat,
-                            nonlinear = lp ~ s(x1) + z + s(x2))
+                            nonlinear = lp ~ s(x1) + z + s(x2, by = x3))
   expect_equal(standata$nb_lp_1, 1)
   expect_equal(as.vector(standata$knots_lp_2), 8)
   expect_equal(dim(standata$Zs_lp_1_1), c(10, 8))
